//  Demo 9: MarchingSquares.java
//
//     A Simple Java Application to Use the Squares Class
//
//             by Charles W. Neville
//      Copyright Charles W. Neville, July, 1999
//
// Two columns of Squares which march a bit like a marching band.
//
// Another illustration of graphics double buffering to avoid animation 
// flicker.  Illustrates methods of the Squares class.

// Handling two columns of Squares at once is difficult and inflexible
// (not easily modified) in this application.  Later, we will see how
// to deal with two or more objects much more easily using Threads
// (multi-tasking).

// Note: Same as the CS 151 Demo MarchingSquaresApplet.java, except this
//       is an application.

import java.awt.*;
import java.applet.Applet;
import Square;  // we are going to do things with individual
                // square objects, so, by contrast with
                // MySquaresApplet.java, we have to import the
                // Square class
import Squares;
import CloseAndExit;

public class MarchingSquares extends Frame {

   private Squares s, t;

   private Graphics gBuffer;   // Graphics buffer for off-screen drawing
   private Image imageBuffer;  // need an Image object to create our
                               // own graphics object, for obscure
                               // Java reasons that only Sun understands

   // Constructor: as usual, initializes variables
   public MarchingSquares() {

     // two columns of 10 20 by 20 squares spaced 60
     // pixels apart
     s = new Squares(50, 50, 20, Color.red, 60, 5);
     t = new Squares(250, 80, 20, Color.blue, 60, 5);

     addWindowListener(new CloseAndExit());   // as usual


   } // Constructor MarchingSquares()

   // init(): creates the Graphics buffer object.  Remember, can't
   //         do this in our Constructor, because we actually have
   //         to exist before the createImage method will work.
   public void init() {

     // create the gBuffer Graphics buffer object
     //   two step process, more Java obscurity
     imageBuffer = createImage(400, 400);  // 400 by 400 pixels

     // we have to wait while the system creates the imageBuffer,
     // or we will simply get a null pointer.
     while(imageBuffer == null) {

        try {
          Thread.sleep(50);  // 50 ms pause
        }
        catch (InterruptedException e) {
          // do nothing
        }
     }

     // and finally, we can get our graphics buffer!
     gBuffer = imageBuffer.getGraphics();

     repaint();

   } // init()

   // march across screen n pixels to right and left
   public void marchAcross(Graphics g, int n) {

     Color backgroundColor = getBackground();

     for(int i = 1; i <= n; i++) {

        // move -- do drawing offscreen on the Graphics
        //         buffer object
        s.draw(gBuffer, backgroundColor);  // hide squares
        t.draw(gBuffer, backgroundColor);  // hide squares
        s.move(1, 0);
        t.move(-1, 0);
        s.draw(gBuffer);
        t.draw(gBuffer);

        //draw completed image on the screen
        g.drawImage(imageBuffer, 0, 0, this);

        try {
          Thread.sleep(50);  // 50 ms pause
        }
        catch (InterruptedException e) {
          // do nothing
        }

     } // for
   } // marchAcross

   // pulsate n times
   public void pulsate(Graphics g, int n) {

     Color backgroundColor = getBackground();
 
     for(int i = 1; i <= n; i++) {

        s.draw(gBuffer, backgroundColor);  // hide squares
        t.draw(gBuffer, backgroundColor);  // hide squares 
        s.expand(-5);
        t.expand(-5);
        s.draw(gBuffer);
        t.draw(gBuffer);

        //draw completed image on the screen
        g.drawImage(imageBuffer, 0, 0, this);

        try {
          Thread.sleep(500);  // 500 ms pause
        }
        catch (InterruptedException e) {
          // do nothing
        }

        s.draw(g, backgroundColor);  // hide squares again
        t.draw(g, backgroundColor);  // hide squares again
        s.expand(5);
        t.expand(5);
        s.draw(g);
        t.draw(g);

        try {
          Thread.sleep(500);  // 500 ms pause
        }
        catch (InterruptedException e) {
          // do nothing
        }

     } // for 
   } // pulsate()

   public void paint(Graphics g) {

      // don't do anything unless imageBuffer is non-null
      if (imageBuffer == null) return;

      for(int i = 1; i <= 25; i++) {

        marchAcross(g, 8);
        pulsate(g, 4);
      }

   } // paint()

   public static void main(String[] args) {

     MarchingSquares ms = new MarchingSquares();

     ms.setSize(400, 400);
     ms.setVisible(true);
     ms.init();

   } // main()

} // class MarchingSquares
        
